/* polycheck.c - test polydiv correctness.
   Written 2002 by Martin Renold <martinxyz@gmx.ch> 
   This code is in the public domain. */
#include "polydiv.h"
#include <assert.h>
#include <stdio.h>
#include <string.h>

/* wether to try dividing by zero... see below. */
#define NUM_CHECKS 3

#define MAX 6

void PolyPrint(float * p, int length)
{
  int j;
  printf("(");
  for (j=length-1; j>=0; j--) printf("%s%5.1f*x^%d ", p[j]<0?" ":"+", p[j], j);
  printf(")");
}

int main() {
  int i, errors;
  float p[MAX], q[MAX], res[MAX], rem[MAX];
  float eres[MAX], erem[MAX];	/* expected result */
  errors = 0;
  for (i=0; i<NUM_CHECKS; i++) {
    bzero(p, MAX*sizeof(float));
    bzero(q, MAX*sizeof(float));
    bzero(eres, MAX*sizeof(float));
    bzero(erem, MAX*sizeof(float));
    switch (i) {
    case 0:
      /* from a math page in the internet */
      p[3] = 1; p[2] = -3; p[1] = -10; p[0] = 24;
      q[1] = 1; q[0] = -2;
      eres[2] = 1; eres[1] = -1; eres[0] = -12;
      /* no remainder expected. */
      break;
    case 1:
      /* from another math page in the internet */
      p[5] = 9; p[4] = 12; p[3] = 10; p[2] = 4; p[1] = 4; p[0] = 2;
      q[2] = 3; q[1] = 2; q[0] = 1;
      eres[3] = 3; eres[2] = 2; eres[1] = 1;
      erem[1] = 3; erem[0] = 2;
      break;
    case 2:
      /* a simpler one. */
      p[5] = -1.2; p[4] = 8; p[3] = 0; p[2] = 2; p[1] = 1; p[0] = 5;
      q[0] = 0.5;
      eres[5] = -2.4; eres[4] = 16; eres[3] = 0; eres[2] = 4; eres[1] = 2; eres[0] = 10;
      break;
    case 3:
      /* 0 : 0 */
      break;
    case 4:
      /* (x^2 + 4x + 2) : 0 */
      p[2] = 1;
      p[1] = 4;
      p[0] = 2;
      break;
    }
    printf("  ");
    PolyPrint(p, MAX);
    printf("\n: ");
    PolyPrint(q, MAX);
    printf("\n= ");
    PolyDiv(p, q, res, rem, MAX);
    PolyPrint(res, MAX);
    printf("\nr ");
    PolyPrint(rem, MAX);
    {
      int j, correct;
      correct = 1;
      for (j=0; j<MAX && correct; j++) {
	if (res[j] != eres[j]) correct = 0;
	if (rem[j] != erem[j]) correct = 0;
	/* bad code; sign missing
	if (res[j] < eres[j]*0.99 || res[j] > eres[j]*1.01) correct = 0;
	if (rem[j] < erem[j]*0.99 || rem[j] > erem[j]*1.01) correct = 0;
	*/
      }
      if (correct) {
	printf("\n(CORRECT)\n\n");
      } else {
	printf("\n(ERROR!!!)\n");
	printf("Expected result:\n");
	printf("= ");
	PolyPrint(eres, MAX);
	printf("\nr ");
	PolyPrint(erem, MAX);
	printf("\n\n");
	errors++;
      }
    }
  }
  printf("%d errors.\n", errors);
  return errors;
}

