/* polydiv.c - do the polynom division.
   Written 2002 by Martin Renold <martinxyz@gmx.ch> 
   This code is in the public domain. */
#include <string.h>
#include <assert.h>
#include "polydiv.h"

void PolyDiv(const float * polynom, const float * quotient, float * result, float * remainder, int length)
{
  int bigQuot; /* last non-zero coefficient of the quotient */
  int bigRem;  /* same for the remainder */
  int i;
  if (length == 0) return;
  assert(length > 0);
  bzero(result, sizeof(float)*length);
  memcpy(remainder, polynom, sizeof(float)*length);
  /* Well, that's it ;-) */
  bigQuot = -1;
  for (i=0; i<length; i++) { /* OPTIMIZE */
    if (quotient[i]) bigQuot = i;
  }
  if (bigQuot == -1) {
    /* Polynom division by zero! */
    /* Hehe, get the right FEELING... */
    bigQuot = 0/0;
  }
  bigRem = length-1; /* worst case; the algo will skip. */
  while (bigRem >= bigQuot) {
    if (remainder[bigRem] != 0) {
      int resQuot = bigRem - bigQuot;
      int i;
      /* OPTIMIZE: cache the result; don't the substraction where we
	 know it becomes zero */
      result[resQuot] = remainder[bigRem]/quotient[bigQuot];
      /* subtract */
      for (i=0; i<=bigQuot; i++) {
	remainder[i+resQuot] -= result[resQuot] * quotient[i];
      }
    }
    /* This condition fails most likely when there wasn't really
       enough memory allocated. */
    assert(remainder[bigRem] == 0);
    bigRem--;
  }
}
